#!/usr/bin/perl -w
# Takes file of RNAfold output.  Extract 20mers that meet miRNA foldback requirements

$file =$ARGV[0];
$out = $ARGV[1];

use lib "/home2/mjonesrh/miRcheck";
use  miRcheck;
$BLOCK = 20; # size fragment to consider as miRNA

#miRcheck parameters
$MAX_UNPAIR = 5; #lower than miRcheck defualt
$MAX_STAR_UNPAIR = 6;
$MAX_SIZEDIFFERENCE = 3;
$MAX_MIR_GAP = 2; #lower than miRcheck defualt
$MAX_STAR_GAP = 3;
$MIN_FBACK_SIZE = 60; 
$MAX_MIR_AS_BULGE = 1; #lower than miRcheck defualt
$BP_EXTENSION = 4;
$MIN_UNPAIR = 1; # minimum number of unpair bases in acceptable extended mirs/mir*s

# parameters for checking sequence composition
$MAX_ONE_NT = 12;
$MAX_TWO_NT = 17;


open(O,">$out");
open(F,$file);
while(<F>){
    if (/>(\S+)/){$id = $1;}
    elsif (/(\S+) \(\S+\)/){
	$fold = $1;
	foreach $pos ($BP_EXTENSION-1 ..length($subseq)-$BLOCK - $BP_EXTENSION){
	    $hseq = substr($subseq,$pos,$BLOCK);
	    if(not(check_comp($hseq))){next;}  # removes mono- and dinucleotide sequences
#	    print "$pos $hseq\n";
	    ($a,$b) = (0,0);
	    ($fback,$a,$b)  = miR_check($fold,$pos,$pos+$BLOCK,  
				"-unpair",$MAX_UNPAIR,
				"-star_unpair",$MAX_STAR_UNPAIR,
				"-size_diff",$MAX_SIZEDIFFERENCE,
				"-mir_bulge",$MAX_MIR_GAP,
				"-star_bulge",$MAX_STAR_GAP,
				"-fback_min",$MIN_FBACK_SIZE,
				"-ass",$MAX_MIR_AS_BULGE,
				"-min_unpair",$MIN_UNPAIR,
				"-bp_ext",$BP_EXTENSION);
	    

	    if ($fback =~ /prime/){
		print O "$hseq $id $pos $fback\n";
	    }
	}
    }
    elsif (/(\S+)/){$subseq = $1; $subseq =~ tr/U/T/;}
}


sub check_comp{
    my ($id,$seq) = @_;
    my ($A,$C,$G,$T,$put,$pyr) = ();

    $seq = uc($seq);
    $A =  count_char('A',$seq);
    $T =  count_char('T',$seq);
    $C =  count_char('C',$seq);
    $G =  count_char('G',$seq);
    $pur = $A+$G;
    $pyr = $C+$T;
    $AC = $A+$C;
    $AT = $A + $T;
    $CG = $C + $G;
    $GT = $G + $T;
#    print "$id A:$A G:$G C:$C T:$T pur:$pur pyr:$pyr\n";
    if ($A){if ($A > $MAX_ONE_NT){return(0);}}
    if ($C){if ($C > $MAX_ONE_NT){return(0);}}
    if ($G){if ($G > $MAX_ONE_NT){return(0);}}
    if ($T){if ($T > $MAX_ONE_NT){return(0);}}
    if ($pur){if ($pur > $MAX_TWO_NT){return(0);}}
    if ($pyr){if ($pyr > $MAX_TWO_NT){return(0);}}
    if ($AC){if ($AC > $MAX_TWO_NT){return(0);}}
    if ($AT){if ($AC > $MAX_TWO_NT){return(0);}}
    if ($CG){if ($AC > $MAX_TWO_NT){return(0);}}
    if ($GT){if ($AC > $MAX_TWO_NT){return(0);}}
    return(1);
}

sub count_char{
    my ($char,$seq) = @_;
    my (@seq,$count,$a) = ();
    $count = 0;
    @seq = split("",$seq);
    foreach $a (@seq){
        if ($a eq $char) {++$count;}
    }
    return($count);
}
