#!/usr/bin/perl -w

use lib "/home2/mjonesrh/miRcheck"; # edit to directory that contains miRcheck.pm 
use miRcheck;

#usage: run_einverted.pl Genome.fa Output_file
# Script to run einverted (emboss) on genome-sized sequences
# Breaks Genome.fa sequences up into fragments (of size $WIN) that overlap by $step
# Calls einverted on each fragment, saving data in Output_file_tmp
# Parses Output_file_tmp, removes redundant inverted repeats (from overlapped fragments)
# Output format: chromosome start_arm1 stop_arm1 start_arm2 stop_arm2 score %complementary


# below are parameters passed to einverted 
$GAP = 6;
$THRESH = 40;
$M = 3;
$MM = 3;
$DIST = 240;

$WIN = 2000;  # size of fragments sent to einverted
$step = 1000; # overlap of fragments sent to einveted
$MIN_PCT = 0.35;  #mimimum percent complementarity in inverted repeats        
$MIN_ARM = 15; #minimum size in inverted repeat

$file = $ARGV[0];
$outfile = $ARGV[1];
open(O, ">$outfile");
open(EI_TMP,">$outfile\_tmp");
$hgenome = hash_FASTA($file);
%genome = %{$hgenome};
foreach $ID (sort keys %genome){
    $size = length($genome{$ID});
#    print "$ID $size\n";
    $x = 1;
    while ($x <= $size-($WIN-$step)){
	$y = $x + $WIN -1;
	$subseq = get_subseq($genome{$ID},$x,$y);
	print EI_TMP ">$ID\_$x\_$y\n";
	run_einverted();
	$x += $step;
    }
}
close(EI_TMP);
parse_einverted("$outfile\_tmp");

sub run_einverted{
    open(SEQ,">$outfile\_seq");
    print SEQ ">$ID\n$subseq\n";
    close(SEQ);
    system("einverted -sequence $outfile\_seq -outfile $outfile\_out -gap $GAP -threshold $THRESH -match $M -mismatch -$MM -maxrepeat $DIST");
    open(OUT,"$outfile\_out");
    while (<OUT>){print EI_TMP;}
    close(OUT);
}


sub parse_einverted{
    my ($EIfile) = @_;
    my (%data) = ();
    $line =5;
    open(TMP_EI,$EIfile);
    
    while(<TMP_EI>){
	++$line;
	if (/>(\S+)_(\S+)_(\S+)/){
	    $chr = $1;
	    $pos = $2;
	    next;
	}
	if (/Score (\d+): (\S+)/){
	    $score = $1; $ratio = $2;
	    $ratio =~ /(\d+)\/(\d+)/;
	    $ratio = $1/$2;
	    if ($ratio >= $MIN_PCT){$line = 0;}
	}
	if ((/^\s+(\d+) \S+ (\d+)/) and ($line ==1)){
	    $beg = $1 +$pos -1;
	    $end = $2 +$pos -1;
	}
	if ((/^\s+(\d+) \S+ (\d+)/) and ($line ==3)){
	    if (($end - $beg >= $MIN_ARM) and ($1 - $2 >= $MIN_ARM)){
		$beg2 = $1 +$pos -1;
		$end2 = $2 +$pos -1;
		$data{"$chr $beg $end $beg2 $end2 $score $ratio"} = 1;
	    }
	}
    }
    foreach $line (sort keys %data){print O "$line\n";}
}
